@@ -19,6 +19,7 @@
 #'                        defaults to 1/sqrt(number_of_functon_calls)
 #' @param chunk_size      Number of function calls to chunk together
 #'                        defaults to 100 chunks per worker or max. 10 kb per chunk
+#' @param timeout         Maximum time in seconds to wait for worker (default: Inf)
 #' @return                A list of whatever `fun` returned
 #' @export
 #'
@@ -37,7 +38,7 @@
 Q = function(fun, ..., const=list(), export=list(), seed=128965,
         memory=NULL, template=list(), n_jobs=NULL, job_size=NULL,
         split_array_by=-1, rettype="list", fail_on_error=TRUE, workers=NULL,
-        log_worker=FALSE, wait_time=NA, chunk_size=NA) {
+        log_worker=FALSE, wait_time=NA, chunk_size=NA, timeout=Inf) {
 
     split_arrays = function(x) {
         if (is.array(x))
@@ -46,15 +47,7 @@ Q = function(fun, ..., const=list(), export=list(), seed=128965,
             x
     }
     iter = lapply(list(...), split_arrays)
-
-    # keep matrices as single columns in df
-    fn = names(formals(fun))
-    if (length(fn) == 1)
-        names(iter) = fn
-    df = data.frame(..placeholder.. = seq_along(iter[[1]]))
-    for (field in names(iter))
-        df[[field]] = iter[[field]]
-    df$..placeholder.. = NULL
+    df = check_args(fun, iter, const)
 
     Q_rows(fun = fun,
            df = df,
@@ -70,5 +63,6 @@ Q = function(fun, ..., const=list(), export=list(), seed=128965,
            workers = workers,
            log_worker = log_worker,
            wait_time = wait_time,
-           chunk_size = chunk_size)
+           chunk_size = chunk_size,
+           timeout = timeout)
 }
